///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      EXCHHDLG.CPP
//
//  Description
//      This file contains the dialogproc that handles messages for
//      the property page extended onto the Tools Options Property Sheet
//
//  Revision: 1.1.00
//
//  Copyright (c) 1998 All rights reserved.
//

#include "EXCHHOOK.H"

///////////////////////////////////////////////////////////////////////////////
// 
//	Forward Prototypes
//
LRESULT APIENTRY AddHandlerDlgProc ( HWND hDlg, UINT message,
	WPARAM wParam, LPARAM lParam );

///////////////////////////////////////////////////////////////////////////////
//    FUNCTION ExchangeHookDlgProc
//
//    Parameters
//    hDlg   - handle to modeless dialog, the property page
//    uMsg   - message
//    wParam - wParam of wndproc, not used
//    lParam - lParam of wndproc, points to NMHDR for notifications
//
//    Purpose
//
//    Return Value
//

BOOL CALLBACK ExchangeHookDlgProc(HWND hDlg, UINT uMsg, 
        WPARAM wParam, LPARAM lParam)
{
 BOOL	bResult;
 BOOL	bMsgResult;
 ULONG  ulCurSel ;
 ULONG	ulItems;
 HKEY    hkHandle;
 DWORD   dwDisposition;
 static HBRUSH hBrush;
 static COLORREF GrayColor;
 static LPNMHDR pnmhdr;
 static HWND hWndPage;

 switch (uMsg)
 {

  case WM_INITDIALOG:
  {
   LOGBRUSH lb;

    GrayColor = (COLORREF)GetSysColor(COLOR_BTNFACE);
    memset(&lb, 0, sizeof(LOGBRUSH));
    lb.lbStyle = BS_SOLID;
    lb.lbColor = GrayColor;
    hBrush = CreateBrushIndirect(&lb);
    return TRUE;
  }
  break;

  case WM_CTLCOLORDLG:
  case WM_CTLCOLORBTN:
  case WM_CTLCOLORSTATIC:
  
    if (hBrush != NULL)
    {
      SetBkColor((HDC)wParam, GrayColor);
      return (BOOL)hBrush;
    }

  break;

  case WM_DESTROY:
  {
   if (hBrush != NULL)
        DeleteObject(hBrush);
   return TRUE;
  }
  
  case WM_COMMAND:
  {
		switch (LOWORD(wParam))
		{
			case IDC_ADD:
			{

				//Clear out the handler name
				szHandlerName[0] = '\0' ;
				
				//Open the dialog for the new item
				bResult = DialogBox (
					ghInstDLL,
					(LPSTR)MAKEINTRESOURCE (IDD_DEFINE_MESSAGEHANDLER),
					hDlg,
					(DLGPROC)AddHandlerDlgProc);

				//Update the listbox with the new name
				if ( bResult )
				{
					//Add an empty item to the dialog
					ulCurSel = SendMessage( GetDlgItem ( hDlg, IDC_LIST1 ),
						LB_INSERTSTRING, -1,
						(LPARAM)szHandlerName );

					//Select the new item
					SendMessage( GetDlgItem ( hDlg, IDC_LIST1 ),
						LB_SETCURSEL, ulCurSel, 0);
				
				}
				bMsgResult = FALSE;
				break;

			}

			case IDC_REMOVE:
			{

				//Find the highlighted item
				ulCurSel = SendMessage( GetDlgItem ( hDlg, IDC_LIST1 ),
					LB_GETCURSEL, 0, 0);

				//Get the handler name
				SendMessage ( GetDlgItem ( hDlg, IDC_LIST1 ),
					LB_GETTEXT, ulCurSel, (LPARAM)szHandlerName );

				//Delete that item from the listbox
				ulItems = SendMessage ( GetDlgItem ( hDlg, IDC_LIST1 ),
					LB_DELETESTRING, ulCurSel, 0 ) ;

				//Delete that item from the registry
				dwDisposition = RegistryOpen ( 
					szRootKey,
					&hkHandle ) ;
				RegistryDelete ( hkHandle, (LPCTSTR)szHandlerName ) ;
				RegistryClose ( hkHandle ) ;

				//Make sure we don't try to highlight a non-existant row
				if ( ulCurSel > ulItems - 1 )
				{
					//Highlight the last item
					SendMessage( GetDlgItem ( hDlg, IDC_LIST1 ),
						LB_SETCURSEL, ulItems - 1, 0);
				}
				else
				{
					//Highlight the item below the one deleted
				    SendMessage( GetDlgItem ( hDlg, IDC_LIST1 ),
						LB_SETCURSEL, ulCurSel, 0);
				}

				bMsgResult = FALSE;
				break;

			}
	  
			case IDC_PROPERTIES:
			{
			
				//Find the highlighted item
				ulCurSel = SendMessage( GetDlgItem ( hDlg, IDC_LIST1 ),
					LB_GETCURSEL, 0, 0);

				//Get the handler name
				SendMessage ( GetDlgItem ( hDlg, IDC_LIST1 ),
					LB_GETTEXT, ulCurSel, (LPARAM)szHandlerName );

				//Open the dialog to edit it
				bResult = DialogBox (
					ghInstDLL,
					(LPSTR)MAKEINTRESOURCE (IDD_DEFINE_MESSAGEHANDLER),
					hDlg,
					(DLGPROC)AddHandlerDlgProc);

				bMsgResult = FALSE;
				break;

			}
			default:
				bMsgResult = FALSE;
				break;

		}
		bMsgResult = FALSE;
		
  }
  break;

  case WM_NOTIFY:
  {
   pnmhdr = ((LPNMHDR) lParam);

   switch ( pnmhdr->code)
   {
    case PSN_KILLACTIVE:
        bMsgResult = FALSE;  // allow this page to receive PSN_APPLY
        break;

    case PSN_SETACTIVE:
		// initialize controls
		RegistryList( hDlg ) ;
        hWndPage = pnmhdr->hwndFrom;   // to be used in WM_COMMAND
        bMsgResult = FALSE;  // accepts activation
        break;
    case PSN_APPLY:
		//accept changes
        bMsgResult = PSNRET_NOERROR;
        break;

    case PSN_HELP:                                              
		bMsgResult = WinHelp (hDlg, "ExchangeHook.HLP", HELP_CONTENTS,(DWORD)0);
        break;

   default:
        bMsgResult = FALSE;
        break;
   }  // switch

    SetWindowLong( hDlg, DWL_MSGRESULT, bMsgResult);
    break;
  
  }   // case WM_NOTIFY

  default:
    bMsgResult = FALSE;
    break;

 }  // switch


 return bMsgResult;
}

///////////////////////////////////////////////////////////////////////////////
//    FUNCTION DlgFnAddHandler
//
//    Parameters
//    hDlg   - handle to dialog box
//    uMsg   - message
//    wParam - wParam of wndproc
//    lParam - lParam of wndproc
//
//    Purpose
//		Handles the events on the dialog box for
//		editing message handlers		
//
//    Return Value
//
LRESULT APIENTRY AddHandlerDlgProc ( HWND hDlg, UINT message,
	WPARAM wParam, LPARAM lParam )
{

BOOL	bMsgResult;
HKEY    hkHandle;
DWORD   dwDisposition;
CHAR	szHandlerKey[120] = TEXT ( "" ) ;

	switch (message)
	{

		case WM_INITDIALOG:
			{
			
				//Initialize everything

				lstrcat(szHandlerKey, szRootKey) ;
				lstrcat(szHandlerKey, "\\") ;
				lstrcat(szHandlerKey, szHandlerName) ;
				
				dwDisposition = RegistryOpen ( 
					szHandlerKey,
					&hkHandle ) ;
				RegistryGet ( hkHandle ) ;
				RegistryClose ( hkHandle ) ;

				// initialize controls
				
				SendDlgItemMessage(hDlg, IDC_CONFIRMDIALOG, BM_SETCHECK, dwConfirmDialog, 0L);
				SendDlgItemMessage(hDlg, IDC_PREVENTOPEN, BM_SETCHECK, dwPreventOpen, 0L);

				SendDlgItemMessage(hDlg, IDC_DEBUG, BM_SETCHECK, dwDebug, 0L);

				SetDlgItemText(hDlg, IDC_EDIT_NAME, szHandlerName);
				SetDlgItemText(hDlg, IDC_EDIT_SENDERNAME, szSenderEmailAddress);
				SetDlgItemText(hDlg, IDC_EDIT_FILEPATH, szFilePath);

				//SetFocus to ourself
				SetFocus ( hDlg );

				bMsgResult = FALSE;
				break;
			}

		case WM_COMMAND:

			bMsgResult = FALSE;

			switch (LOWORD(wParam))
			{
				case IDC_OK:
				{
					if (HIWORD (wParam) == BN_CLICKED)
					{
						
						UINT	uLen ;

						// get user input

						uLen = SendDlgItemMessage(hDlg, IDC_EDIT_NAME, WM_GETTEXT, (WPARAM)HandlerNameSize, (LPARAM)szHandlerName);
						if ( uLen = 0 )
						{
							ErrorMessageBox(hDlg, 0, szMessageBoxTitle, 
								"The handler must have a name.");
							bMsgResult = FALSE;
							break;
						}

						dwConfirmDialog = SendDlgItemMessage(hDlg, IDC_CONFIRMDIALOG, BM_GETCHECK, 0, 0L);
						dwPreventOpen = SendDlgItemMessage(hDlg, IDC_PREVENTOPEN, BM_GETCHECK, 0, 0L);
						SendDlgItemMessage(hDlg, IDC_EDIT_SENDERNAME, WM_GETTEXT, (WPARAM)SenderEmailAddressSize, (LPARAM)szSenderEmailAddress);
						SendDlgItemMessage(hDlg, IDC_EDIT_FILEPATH, WM_GETTEXT, (WPARAM)FilePathSize, (LPARAM)szFilePath);
						dwDebug = SendDlgItemMessage(hDlg, IDC_DEBUG, BM_GETCHECK, 0, 0L);

						lstrcat(szHandlerKey, szRootKey) ;
						lstrcat(szHandlerKey, "\\") ;
						lstrcat(szHandlerKey, szHandlerName) ;
						
						//Update the registry
						dwDisposition = RegistryOpen ( 
							szHandlerKey,
							&hkHandle ) ;
						RegistrySet ( hkHandle ) ;
						RegistryClose ( hkHandle ) ;
						
						EndDialog (hDlg, TRUE);
					}
					break;
				}

				case IDC_CANCEL:
				{
					if (HIWORD (wParam) == BN_CLICKED)
					{
						EndDialog (hDlg, FALSE);
					}
					break;
				}

				case IDC_DLGHELP:
				{
					if (HIWORD (wParam) == BN_CLICKED)
					{
					WinHelp (hDlg, "ExchangeHook.HLP", HELP_CONTENTS,(DWORD)0);
					}
					break;
				}

				case IDC_CONFIRMDIALOG:
				{
					if (HIWORD (wParam) == EN_CHANGE)
					{
						//
					}
					bMsgResult = FALSE;
					break;
				}

				case IDC_PREVENTOPEN:
				{
					if (HIWORD (wParam) == EN_CHANGE)
					{
						//
					}
					bMsgResult = FALSE;
					break;
				}
			
				case IDC_DEBUG:
				{
					if (HIWORD (wParam) == EN_CHANGE)
					{
						//
					}
					bMsgResult = FALSE;
					break;
				}

				case IDC_EDIT_NAME:
				{
					if (HIWORD (wParam) == EN_CHANGE)
					{
						//
					}
					bMsgResult = FALSE;
					break;
				}

				case IDC_EDIT_SENDERNAME:
				{
					if (HIWORD (wParam) == EN_CHANGE)
					{
					//
					}
					bMsgResult = FALSE;
					break;
				}

				case IDC_EDIT_FILEPATH:
				{
					if (HIWORD (wParam) == EN_CHANGE)
					{
					//
					}
					bMsgResult = FALSE;
					break;
				}

				case IDC_BUTTON_SENDERNAME:
				{
					if (HIWORD (wParam) == BN_CLICKED)
					{
						LPADRLIST		pal = NULL;
						ADRPARM			ap = { 0 };
						HRESULT			hr ;
						ULONG			ulUIParam = (ULONG)hDlg;
						LPSPropValue	lpvalT;
						UINT ip;

						ap.ulFlags = AB_RESOLVE |
							ADDRESS_ONE |
							DIALOG_OPTIONS |
							DIALOG_MODAL |
							fMapiUnicode;
						ap.lpszCaption = TEXT("Select Addressee");
						ap.cDestFields = 1;
						hr = lpAdrbook->Address(
							&ulUIParam,
							&ap,
							&pal);
						if (FAILED(hr))
						{
							bMsgResult = FALSE;
							break;
						}

						lpvalT = pal->aEntries[0].rgPropVals;
						for (ip = 0; ip < pal->aEntries[0].cValues; lpvalT++, ip++)
						{
							if (lpvalT->ulPropTag == PR_EMAIL_ADDRESS )
							{
								SetDlgItemText ( hDlg, IDC_EDIT_SENDERNAME, lpvalT->Value.lpszA );
							}
						}
					
					}
					bMsgResult = FALSE;
					break;
				}

				case IDC_BUTTON_FILEPATH:
				{
					if (HIWORD (wParam) == BN_CLICKED)
					{
						TCHAR    szFileName  [MAX_PATH];
						TCHAR    szInitialFileName  [MAX_PATH] = {""};
						OPENFILENAME		OFN ;

						OFN.lStructSize = sizeof (OPENFILENAME);
						OFN.hwndOwner = GetParent(hDlg);
						OFN.hInstance = GetModuleHandle(NULL);
						OFN.lpstrFilter = "Program Files\0*.exe\0";
						OFN.lpstrCustomFilter = (LPTSTR)NULL;
						OFN.nMaxCustFilter = 0L;
						OFN.nFilterIndex = 1L;
						OFN.lpstrFile = szInitialFileName;
						OFN.nMaxFile = _MAX_PATH;
						OFN.lpstrFileTitle = szFileName;
						OFN.nMaxFileTitle = _MAX_PATH;
						OFN.lpstrInitialDir = NULL;
						OFN.lpstrTitle = "Program to Run" ;
						OFN.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST;
						OFN.nFileOffset = 0;
						OFN.nFileExtension = 0;
						OFN.lpstrDefExt = "exe";
						OFN.lCustData = 0; 
						OFN.lpfnHook = NULL;
						OFN.lpTemplateName = NULL;

						// Display the Open dialog box. 
						if (GetOpenFileName(&OFN))
						{
							SetDlgItemText ( hDlg, IDC_EDIT_FILEPATH, OFN.lpstrFile );
						}
					}
					bMsgResult = FALSE;
					break;
				}

				default:
				{
					bMsgResult = FALSE;
					break;
				}

			} /* end switch wparam */
			break;

		default:
		{
			bMsgResult = FALSE;
			break;
		}

	} /* end switch message */

	return 	bMsgResult ;

}
